VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsParser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Private class properties
Private mStrSeparators As String        ' A string holding all valid token separators.
Private mStrTerminators As String       ' A string holding all valid line terminators.
Private mStrParseString As String       ' String to parse.
Private mLngPP As Long                  ' Current parser position.

Public Property Let ParseString(str As String)
    ' Reset the parse pointer and set the mStrParseString variable.
    mLngPP = 1
    mStrParseString = str
End Property

Public Property Let Separators(str As String)
    ' Reset the parse separators.
    mStrSeparators = str
End Property

Public Property Let Terminators(str As String)
    ' Reset the parse terminators.
    mStrTerminators = str
End Property

Public Function ParseNumberSequence(colNumbers As Collection) As Boolean
    ' Parse a sequence of numbers and store in a collection.
    Dim dblVal As Double

    ParseNumberSequence = False     ' Assume failure.
    ParseWhitespace                 ' Ignore white space at the beginning.
        
    ' A sequence must begin with at least one number.
    If ParseNumber(dblVal) = True Then
        colNumbers.Add dblVal
        ParseNumberSequence = True
        
        ' Now look for more elements in the sequence.
        If ParseSeparator() = True Then ParseNumberSequence = ParseNumberSequence(colNumbers)
    End If

End Function

Public Function ParseNumber(dblVal As Double) As Boolean
    ' Parse a number.  Always returns a double.
    Dim I1 As Long, strDigSeq As String
    
    ' First look for an integer.
    If ParseInteger(I1) = True Then
        ' If get this far then the procedure is successful.
        ParseNumber = True
        
        ' If a decimal place found then look for decimal values as a positive integer.
        ' Build the resulting double.
        If ParseLiteral(".") = True And ParseDigitSequence(strDigSeq) = True Then
            dblVal = CDbl(CStr(I1) & "." & strDigSeq)
        Else
            dblVal = CDbl(I1)
        End If
    End If

End Function

Public Function ParseInteger(lngVal As Long) As Boolean
    ' Parse positive or negative integers.
    Dim lngMultiplier As Long, strDigitSequence As String
    
    ParseInteger = False                ' Assume failure.
    lngMultiplier = 1&                  ' Assume the integer is positive.
    
    ' Look for a leading negative or positive sign.
    If ParseLiteral("-") = True Then
        lngMultiplier = -1&
    ElseIf ParseLiteral("+") Then
        ' Do Nothing
    End If
    
    ' Get a sequence of digits
    If ParseDigitSequence(strDigitSequence) = True Then
    
        If Not IsNumeric(strDigitSequence) Then Exit Function
        lngVal = CLng(strDigitSequence)
        
        ' Adjust sign.
        lngVal = lngMultiplier * lngVal
        ParseInteger = True
    
    End If
    
End Function

Public Function ParseDigitSequence(strDigitSequence As String) As Boolean
    ' Parse and return a string of sequential digits.
    Dim lngStartPP As Long, intLen As Integer
    
    ParseDigitSequence = False          ' Assume failure.
    lngStartPP = mLngPP                 ' Save initial location.
    
    ' Recognize at least one digit.
    If ParseDigit() = True Then
        ' Recognize additional digits on the sequence.
        While ParseDigit() = True:  Wend
        
        ' Return the sequence
        intLen = mLngPP - lngStartPP
        strDigitSequence = Mid$(mStrParseString, lngStartPP, intLen)
        ParseDigitSequence = True
    End If
    
End Function

Public Function ParseDigit() As Boolean
    ' Determine if the current character is a valid digit
    Dim strChar As String
    
    ParseDigit = False
    If EOF() = True Then Exit Function
    
    strChar = Mid$(mStrParseString, mLngPP, 1)
    If InStr("0123456789", strChar) Then
        mLngPP = mLngPP + 1
        ParseDigit = True
    End If

End Function

Public Function ParseLetter() As Boolean
    ' Determine if the current character is a valid letter
    Dim intChar As Integer
    
    ParseLetter = False     ' Assume failure.
    If EOF() = True Then Exit Function

    ' Use the ASCII code to determine if the character is a letter
    intChar = Asc(Mid$(mStrParseString, mLngPP, 1))
    If (intChar >= 65 And intChar <= 90) Or (intChar >= 97 And intChar <= 122) Then
        mLngPP = mLngPP + 1
        ParseLetter = True
    End If
    
End Function

Public Function ParseIdentifier(strIdent As String) As Boolean
    ' Parse an identfier and return it if found.
    Dim lngStartPP As Long, intLen As Integer
    
    ParseIdentifier = False             ' Assume failure.
    lngStartPP = mLngPP                 ' Save initial location.
    
    ' Identfiers must begin with a letter.
    If ParseLetter() = True Then
        ' Continue incrementing until the next characters is not a digit or letter.
        While ParseLetter() = True Or ParseDigit() = True: Wend
        
        ' Return the new identfier.
        intLen = mLngPP - lngStartPP
        strIdent = Mid$(mStrParseString, lngStartPP, intLen)
        ParseIdentifier = True
    End If
        
End Function

Public Function ParseDelimitedToken(strToken As String) As Boolean
    ' Get the next token by looking for a separator, terminator or end of the parse string.
    Dim intLen As Integer
    
    ParseDelimitedToken = False     ' Assume failure
    
    ' Continue while not the end of the parse string, a separator, or a terminator.
    intLen = 0
    Do
        ' Handle each case separately to prevent unwanted behavior.
        If EOF() = True Then
            strToken = Mid$(mStrParseString, mLngPP - intLen, intLen)
            ParseDelimitedToken = True
            Exit Do
        ElseIf ParseSeparator() = True Then
            strToken = Mid$(mStrParseString, mLngPP - intLen - 1, intLen)
            ParseDelimitedToken = True
            Exit Do
        ElseIf ParseTerminator() = True Then
            strToken = Mid$(mStrParseString, mLngPP - intLen - 1, intLen)
            ParseDelimitedToken = True
            Exit Do
        End If
        
        intLen = intLen + 1
        mLngPP = mLngPP + 1
    Loop
        
End Function

Public Function ParseLiteral(strLit As String) As Boolean
    ' Look for an exact match to the given string.
    Dim intLLen As Integer
    
    ParseLiteral = False        ' Assume failure
    intLLen = Len(strLit)       ' Get the length of the literal string.
    
    ' Look for the literal string.
    If Mid$(mStrParseString, mLngPP, intLLen) = strLit Then
        mLngPP = mLngPP + intLLen
        ParseLiteral = True
    End If

End Function

Public Function ParseSeparator() As Boolean
    ' Look for the next character to be a valid separator character.
    Dim strChar As String
    
    ParseSeparator = False
    If EOF() = True Then Exit Function

    strChar = Mid$(mStrParseString, mLngPP, 1)
    If InStr(mStrSeparators, strChar) > 0 Then
        mLngPP = mLngPP + 1
        ParseSeparator = True
    End If
    
End Function

Public Function ParseTerminator() As Boolean
    ' Look for the next character to be a valid terminator character.
    Dim strChar As String
    
    ParseTerminator = False
    If EOF() = True Then Exit Function
    
    ' Match the terminator character
    strChar = Mid$(mStrParseString, mLngPP, 1)
    If InStr(mStrTerminators, strChar) > 0 Then
        mLngPP = mLngPP + 1
        ParseTerminator = True
    End If

End Function

Public Sub ParseWhitespace()
    ' Discard any whitespace found.
    Dim strChar As String
    
    strChar = Mid$(mStrParseString, mLngPP, 1)
    While strChar = " "
        mLngPP = mLngPP + 1
        strChar = Mid$(mStrParseString, mLngPP, 1)
    Wend
    
End Sub

Public Function EOF() As Boolean
    ' True, if past end of ParseString
    EOF = False
    If mLngPP > Len(mStrParseString) Then EOF = True
End Function

Public Sub SyntaxError(Msg As String)
    ' Display a syntax error with an indication of the problem.
    Dim strMsg As String
    
    strMsg = "Syntax Error at position " & CStr(mLngPP) & ".  " & Msg & "." & vbLf
    strMsg = strMsg & Left$(mStrParseString, mLngPP - 1) & "<here>"
    strMsg = strMsg & Right$(mStrParseString, Len(mStrParseString) - mLngPP + 1) & vbLf
    
    MsgBox strMsg
End Sub

