VERSION 5.00
Object = "{248DD890-BB45-11CF-9ABC-0080C7E7B78D}#1.0#0"; "MSWINSCK.OCX"
Begin VB.Form frmWinsockCtrl 
   Caption         =   "Winsock VI Controller"
   ClientHeight    =   1020
   ClientLeft      =   2385
   ClientTop       =   2010
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   PaletteMode     =   1  'UseZOrder
   ScaleHeight     =   1020
   ScaleWidth      =   4680
   Begin MSWinsockLib.Winsock sckVI 
      Left            =   120
      Top             =   480
      _ExtentX        =   741
      _ExtentY        =   741
      _Version        =   393216
   End
   Begin VB.ComboBox cmbSend 
      Height          =   315
      Left            =   120
      TabIndex        =   1
      Top             =   120
      Width           =   3675
   End
   Begin VB.CommandButton btnSend 
      Caption         =   "Send"
      Default         =   -1  'True
      Height          =   375
      Left            =   3840
      TabIndex        =   0
      Top             =   120
      Width           =   735
   End
End
Attribute VB_Name = "frmWinsockCtrl"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

' The state of the Controller
Private mIntState  As Integer

' State constants
Const conIdle As Integer = 0                ' Controller is idle
Const conRequestedAnalyze As Integer = 1    ' Controller requested the VI to Analyze
Const conRequestedPause As Integer = 2      ' Controller requested the VI to Pause
Const conRequestedResume As Integer = 3     ' Controller requested the VI to Resume
Const conRequestedAbort As Integer = 4      ' Controller requested the VI to Abort
Const conRequestedOff As Integer = 5        ' Controller requested the VI to turn Off
Const conRequestedUnknown As Integer = 6    ' Controller requested the VI to perform an unknown task
Const conCollectingData As Integer = 7      ' Controller is collecting data from the VI

' Buffer to hold data received on input channel
Private mStrInputBuffer As String

' Object variable for Parser class instance
Private mParser As clsParser

Private Sub Form_Load()
                        ' Connect to the VI over the network.
    If sckVI.State <> sckClosed Then sckVI.Close
    With sckVI          ' Set up the Winsock control
        .RemoteHost = "127.0.0.1" ' ... or enter a name
        .RemotePort = 1234
        .Connect        ' Connect
    End With
                        ' Create and setup new clsParser class
    Set mParser = New clsParser
    mParser.Separators = ","
    mParser.Terminators = vbCr
    
                        ' Fill the combobox
    cmbSend.AddItem "Analyze"
    cmbSend.AddItem "Pause"
    cmbSend.AddItem "Resume"
    cmbSend.AddItem "Abort"
    cmbSend.AddItem "Off"
    
                        ' Initialize the controller state
    mIntState = conIdle
End Sub

Private Sub ProcessInputBuffer()
    Dim strMessage As String
    
    ' Get any new data
    If FetchInputData() = False Then Exit Sub
    
    ' Process all complete messages.
    strMessage = GetMessage()
    Do While strMessage <> ""
        ProcessMessage strMessage
        strMessage = GetMessage()
    Loop
        
End Sub

Private Function FetchInputData() As Boolean
    ' Get any new data from the input port and append to Input Queue.
    ' Return true if any found.
    Dim strData As String
    
    FetchInputData = False                  ' Assume no new data.
    
    If sckVI.BytesReceived > 0 Then
        sckVI.GetData strData, vbString
        mStrInputBuffer = mStrInputBuffer & strData
        FetchInputData = True
    End If
    
End Function

Private Function GetMessage() As String
    ' Get an entire message.
    Dim intPosition As Integer              ' Position of the line termination character(s)
    Dim strMessage As String                ' Store a message
                                            ' Error handler
    On Error GoTo GetMessageErrorHandler
    
    strMessage = ""                         ' Look for line termination characters
    intPosition = InStr(mStrInputBuffer, vbCr)
    If intPosition > 0 Then                 ' If found then remove the command
        strMessage = Left$(mStrInputBuffer, intPosition - 1)
                                            ' Keep any remaining data
        mStrInputBuffer = Right$(mStrInputBuffer, Len(mStrInputBuffer) - intPosition)
    End If
    
    GetMessage = strMessage                 ' Return message
    Exit Function
'______________________________
GetMessageErrorHandler:
    Select Case Err.Number
    Case Else
        MsgBox "Error (GetMessage) " & CStr(Err.Number) & ": " & Err.Description
    End Select
End Function

Private Sub ProcessMessage(strMessage As String)
    ' Process Message from a command
    Dim strStatus As String, colData As New Collection
    
    mParser.ParseString = strMessage            ' Set the statement to parse
    
    Select Case mIntState                       ' Switch on the state of the controller
    Case conIdle                                ' When idle, indicate communication was unexpected
        MsgBox "Unexpected communication: " & strMessage
        
    Case conRequestedAnalyze                    ' Controller requested the VI to Analyze
        If ParseStatus(strStatus) = True Then   ' Expecting a status line
            mIntState = conCollectingData       ' Change state to collecting data
        End If
        
    Case conCollectingData                      ' Controller is collecting data
                                                ' Expecting a sequence of data
        If mParser.ParseNumberSequence(colData) = True Then
            MsgBox "Collected " & CStr(colData.Count) & " data points"
            mIntState = conIdle                 ' Change state back to idle
        End If
        
    Case Else                                   ' Anything else returns a status line
        If ParseStatus(strStatus) = True Then   ' Expecting a status line
            If Left(strStatus, 5) = "ERROR" Then MsgBox "Command failed: " & strStatus
            mIntState = conIdle                 ' Change state back to idle
        End If
    End Select
    
End Sub

Private Function ParseStatus(strStatus As String) As Boolean
    ' Parse a status line
    Dim lngCode As Long
    
    With mParser
        ' Look for an "OK"
        If .ParseLiteral("OK") = True Then
                strStatus = "OK"
                ParseStatus = True
        
        ' Look for an error status line, get and return code
        ElseIf .ParseLiteral("Error") = True Then
            .ParseWhitespace
            If .ParseInteger(lngCode) = True Then
                strStatus = "ERROR " & CStr(lngCode)
                ParseStatus = True
            End If
        Else
            ParseStatus = False
        End If
            
    End With
    
End Function

Private Sub btnSend_Click()
    Dim strCommand As String
    
    On Error GoTo btnSendErrorHandler
    
    ' Massage the command string
    strCommand = UCase(Trim(cmbSend.Text))
    
    ' Send contents of the text box over the network connection.
    If sckVI.State <> sckConnected Then
        MsgBox "Error.  Network connection is not open! (State=" & CStr(sckVI.State) & ")"
        Exit Sub
    Else
        sckVI.SendData strCommand & vbCr
    End If
    
    ' Change state based on command to be issued
    If Left$(strCommand, 7) = "ANALYZE" Then
        mIntState = conRequestedAnalyze
    ElseIf strCommand = "PAUSE" Then
        mIntState = conRequestedPause
    ElseIf strCommand = "RESUME" Then
        mIntState = conRequestedResume
    ElseIf strCommand = "ABORT" Then
        mIntState = conRequestedAbort
    ElseIf strCommand = "OFF" Then
        mIntState = conRequestedOff
    Else
        mIntState = conRequestedUnknown
    End If
    
    Exit Sub
'_______________________
btnSendErrorHandler:

    Select Case Err.Number
    Case Else
        MsgBox "Error " & CStr(Err.Number) & ": " & Err.Description
    End Select
End Sub

Private Sub sckVI_DataArrival(ByVal bytesTotal As Long)
    ' Process data received over the network connection.
    ProcessInputBuffer
    
End Sub

Private Sub sckVI_Error(ByVal Number As Integer, Description As String, ByVal Scode As Long, ByVal Source As String, ByVal HelpFile As String, ByVal HelpContext As Long, CancelDisplay As Boolean)
    MsgBox "sckVI error: " & CStr(Number) & " " & Description
End Sub
