VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "GasGauge"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' A reference to the PictureBox on which top draw the gauge
Private WithEvents mPct As PictureBox
Attribute mPct.VB_VarHelpID = -1

' The value currently displayed on the gauge [0, 100]
Private mSngValue As Single

' The value that divides the red and green area [0, 100]
Private mSngSetPoint As Single

Const Pi As Single = 3.14159        ' The constant Pi

Private Sub DrawGauge()
    ' Draw the gauge on the PictureBox
    Dim sngX As Single, sngY As Single
    
    ' Restore the background
    mPct.Cls
    
    ' Draw the needle
    sngX = 0.95 * Cos((100# - mSngValue) * 0.01 * Pi)
    sngY = 0.95 * Sin((100# - mSngValue) * 0.01 * Pi)
    mPct.DrawWidth = 4
    mPct.Line (0#, 0#)-(sngX, sngY), vbBlack
    
End Sub

Public Property Set PictureBox(pctBox As PictureBox)
    Dim sngHeight As Single, sngWidth As Single, sngNewHeight As Single

    ' Set the PictureBox on which the gauge should be drawn
    Set mPct = pctBox

    ' Get PictureBox dimensions
    sngWidth = mPct.Width
    sngHeight = mPct.Height
    
    ' Set the coordinates of the PictureBox.
    ' The x-dimension is always constraining when drawing a circle.
    sngNewHeight = 2.8 * (sngHeight / sngWidth) - 0.8
    'mPct.Scale (-1.4, sngNewHeight + 0.4)-(1.4, -0.4)
    With mPct
        .ScaleLeft = -1.4
        .ScaleWidth = 2.8
        .ScaleTop = sngNewHeight + 0.4
        .ScaleHeight = -(sngNewHeight + 0.8)
    End With

    
    ' Set the background and draw the gauge
    SetBackground
    DrawGauge
    
End Property

Private Sub SetBackground()
    ' Draw the background and save in PictureBox
    Dim sngX As Single, sngY As Single, sngX2 As Single, sngY2 As Single
    Dim I As Integer
    
    mPct.AutoRedraw = True          ' Initiate the persistent graphic
    mPct.BackColor = vbWhite        ' Clear

    mPct.DrawWidth = 2              ' Set graphic properties
    mPct.FillStyle = 0 ' solid
    
    mPct.FillColor = vbGreen
    mPct.Circle (0#, 0#), 1, vbBlack, -0.00001, -0.01 * (100# - mSngSetPoint) * Pi
    mPct.FillColor = vbRed
    mPct.Circle (0#, 0#), 1#, vbBlack, -0.01 * (100# - mSngSetPoint) * Pi, -Pi

    For I = 0 To 100 Step 25            ' Draw ticks
        sngX = Cos((100# - I) * 0.01 * Pi)
        sngY = Sin((100# - I) * 0.01 * Pi)
        sngX2 = 0.9 * Cos((100# - I) * 0.01 * Pi)
        sngY2 = 0.9 * Sin((100# - I) * 0.01 * Pi)
        mPct.Line (sngX, sngY)-(sngX2, sngY2), vbBlack
    Next I

    mPct.Font.Size = 18             ' Draw the 'E' and 'F' indicators
    mPct.CurrentX = -1.3
    mPct.CurrentY = 0.3
    mPct.Print "E"

    mPct.CurrentX = 1.1
    mPct.CurrentY = 0.3
    mPct.Print "F"
    
    mPct.Picture = mPct.Image       ' Store persistent graphic in the Picture property
    mPct.AutoRedraw = False

End Sub

Public Property Get SetPoint() As Single
    ' Return the gauge SetPoint property value
    SetPoint = mSngSetPoint
End Property

Public Property Let SetPoint(sngSetPoint As Single)
    ' Set the value that marks the separation between the green and red areas.
    
    ' Clip to the range [0, 100]
    If sngSetPoint < 0# Then
        mSngSetPoint = 0#
    ElseIf sngSetPoint > 100# Then
        mSngSetPoint = 100#
    Else
        mSngSetPoint = sngSetPoint
    End If
    
    ' Reset the new background and redraw the gauge
    SetBackground
    DrawGauge
    
End Property

Public Property Get Value() As Single
    ' Return the gauge Value property value
    Value = mSngValue
End Property

Public Property Let Value(sngValue As Single)
    ' Set the value to be displayed by the needle on the gauge.
    
    ' Clip to the range [0, 100]
    If sngValue < 0# Then
        mSngValue = 0#
    ElseIf sngValue > 100# Then
        mSngValue = 100#
    Else
        mSngValue = sngValue
    End If
    
    ' Redraw the gauge
    DrawGauge
    
End Property

Private Sub Class_Initialize()
    ' Initialize GasGauge class property values
    mSngSetPoint = 25#
    mSngValue = 0#
End Sub

Private Sub mPct_Paint()
    ' Redraw the gauge
    DrawGauge
End Sub
